@@ -1,6 +1,7 @@
 # appstream-metadata -- lintian check script -*- perl -*-
 
 # Copyright © 2016 Petter Reinholdtsen
+# Copyright © 2017-2018 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,8 +30,9 @@ package Lintian::appstream_metadata;
 # For modaliases, maybe udev rules could give some hints.
 # Check modalias values to ensure hex numbers are using capital A-F.
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie qw(open);
 
 use File::Basename qw(basename);
