@@ -1,6 +1,7 @@
 # deb-format -- lintian check script -*- perl -*-
 
-# Copyright (C) 2009 Russ Allbery
+# Copyright © 2009 Russ Allbery
+# Copyright © 2018 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -17,8 +18,9 @@
 
 package Lintian::deb_format;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use IO::Async::Loop;
