@@ -1,6 +1,6 @@
 # debian/debconf -- lintian check script -*- perl -*-
 
-# Copyright (C) 2001 Colin Watson
+# Copyright © 2001 Colin Watson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 
 package Lintian::debian::debconf;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
-use Lintian::Deb822Parser qw(read_dpkg_control :constants);
+use Path::Tiny;
+
+use Lintian::Deb822Parser qw(parse_dpkg_control_string :constants);
 use Lintian::Relation;
 use Lintian::Util qw($PKGNAME_REGEX);
 
@@ -66,7 +69,64 @@ my $ANY_DEBCONF = Lintian::Relation->new(
           cdebconf-udeb libdebconfclient0 libdebconfclient0-udeb)
     ));
 
-sub always {
+sub source {
+    my ($self) = @_;
+
+    my @binaries = $self->processable->binaries;
+    my @files = map { "$_.templates" } @binaries;
+    push @files, 'templates';
+
+    foreach my $file (@files) {
+        my $dfile = "debian/$file";
+        my $templates_file = $self->processable->patched->resolve_path($dfile);
+        my $binary = $file;
+        $binary =~ s/\.?templates$//;
+        # Single binary package (so @files contains "templates" and
+        # "binary.templates")?
+        if (!$binary && @files == 2) {
+            $binary = $binaries[0];
+        }
+
+        if ($templates_file && $templates_file->is_open_ok) {
+
+            unless ($templates_file->is_valid_utf8) {
+                $self->tag('national-encoding-in-debconf-template',
+                    $templates_file->name);
+                next;
+            }
+
+            my $contents = $templates_file->decoded_utf8;
+
+            my @templates;
+            eval {
+                @templates
+                  = parse_dpkg_control_string($contents,
+                    DCTRL_DEBCONF_TEMPLATE);
+            };
+            if ($@) {
+                chomp $@;
+                $@ =~ s/^internal error: //;
+                $@ =~ s/^syntax error in //;
+                $self->tag('syntax-error-in-debconf-template',"$file: $@");
+                next;
+            }
+
+            foreach my $template (@templates) {
+                if (    exists $template->{template}
+                    and exists $template->{_choices}) {
+                    $self->tag(
+                        'template-uses-unsplit-choices',
+                        "$binary - $template->{template}"
+                    );
+                }
+            }
+        }
+    }
+
+    return;
+}
+
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
@@ -75,59 +135,12 @@ sub always {
 
     my ($seenconfig, $seentemplates, $usespreinst);
 
-    if ($type eq 'source') {
-        my @binaries = $processable->binaries;
-        my @files = map { "$_.templates" } @binaries;
-        push @files, 'templates';
-
-        foreach my $file (@files) {
-            my $dfile = "debian/$file";
-            my $templates_file = $processable->patched->resolve_path($dfile);
-            my $binary = $file;
-            $binary =~ s/\.?templates$//;
-            # Single binary package (so @files contains "templates" and
-            # "binary.templates")?
-            if (!$binary && @files == 2) {
-                $binary = $binaries[0];
-            }
-
-            if ($templates_file and $templates_file->is_open_ok) {
-                my @templates;
-                eval {
-                    @templates
-                      = read_dpkg_control($templates_file->fs_path,
-                        DCTRL_DEBCONF_TEMPLATE);
-                };
-                if ($@) {
-                    chomp $@;
-                    $@ =~ s/^internal error: //;
-                    $@ =~ s/^syntax error in //;
-                    $self->tag('syntax-error-in-debconf-template',"$file: $@");
-                    next;
-                }
-
-                foreach my $template (@templates) {
-                    if (    exists $template->{template}
-                        and exists $template->{_choices}) {
-                        $self->tag(
-                            'template-uses-unsplit-choices',
-                            "$binary - $template->{template}"
-                        );
-                    }
-                }
-            }
-        }
-
-        # The remainder of the checks are for binary packages, so we exit now
-        return;
-    }
-
     my $preinst = $processable->control->lookup('preinst');
     my $ctrl_config = $processable->control->lookup('config');
     my $ctrl_templates = $processable->control->lookup('templates');
 
     if ($preinst and $preinst->is_file and $preinst->is_open_ok) {
-        my $fd = $preinst->open;
+        open(my $fd, '<', $preinst->unpacked_path);
         while (<$fd>) {
             s/\#.*//;    # Not perfect for Perl, but should be OK
             if (   m,/usr/share/debconf/confmodule,
@@ -190,20 +203,23 @@ sub always {
     my (@templates, %potential_db_abuse, @templates_seen);
 
     if ($seentemplates) {
-        eval {
-            # $seentemplates (above) will be false if $ctrl_templates is a
-            # symlink or not a file, so this should be safe without
-            # (re-checking) with -f/-l.
-            @templates
-              = read_dpkg_control($ctrl_templates->fs_path,
-                DCTRL_DEBCONF_TEMPLATE);
-        };
-        if ($@) {
-            chomp $@;
-            $@ =~ s/^internal error: //;
-            $@ =~ s/^syntax error in //;
-            $self->tag('syntax-error-in-debconf-template', "templates: $@");
-            @templates = ();
+
+        if ($ctrl_templates->is_valid_utf8) {
+            my $contents = $ctrl_templates->decoded_utf8;
+            eval {
+                # $seentemplates (above) will be false if $ctrl_templates is a
+                # symlink or not a file, so this should be safe without
+                # (re-checking) with -f/-l.
+                @templates= parse_dpkg_control_string($contents,
+                    DCTRL_DEBCONF_TEMPLATE);
+            };
+            if ($@) {
+                chomp $@;
+                $@ =~ s/^internal error: //;
+                $@ =~ s/^syntax error in //;
+                $self->tag('syntax-error-in-debconf-template',"templates: $@");
+                @templates = ();
+            }
         }
     }
 
@@ -405,7 +421,7 @@ sub always {
         if ($path and $path->is_file and $path->is_open_ok) {
             my ($usesconfmodule, $obsoleteconfmodule, $db_input, $isdefault);
 
-            my $fd = $path->open;
+            open(my $fd, '<', $path->unpacked_path);
             # Only check scripts.
             my $fl = <$fd>;
             unless ($fl && $fl =~ /^\#!/) {
@@ -569,17 +585,21 @@ sub always {
     # the following checks is ignored if the package being checked is debconf
     # itself.
 
-    return if ($pkg eq 'debconf') || ($type eq 'udeb');
+    return
+      if ($pkg eq 'debconf') || ($type eq 'udeb');
 
-    foreach my $filename (sort keys %{$processable->scripts}) {
-        my $path = $processable->installed->resolve_path($filename);
-        next if not $path or not $path->is_open_ok;
-        my $fd = $path->open;
+    my @scripts = grep { $_->is_script } $processable->installed->sorted_list;
+    foreach my $file (@scripts) {
+
+        next
+          unless $file->is_open_ok;
+
+        open(my $fd, '<', $file->unpacked_path);
         while (<$fd>) {
             s/#.*//;    # Not perfect for Perl, but should be OK
             if (   m,/usr/share/debconf/confmodule,
                 or m/(?:Debconf|Debian::DebConf)::Client::ConfModule/) {
-                $self->tag('debconf-is-not-a-registry', $filename);
+                $self->tag('debconf-is-not-a-registry', $file->name);
                 last;
             }
         }
