@@ -1,9 +1,9 @@
 # fields/architecture -- lintian check script (rewrite) -*- perl -*-
 #
-# Copyright (C) 2004 Marc Brockschmidt
+# Copyright © 2004 Marc Brockschmidt
 #
 # Parts of the code were taken from the old check script, which
-# was Copyright (C) 1998 Richard Braakman (also licensed under the
+# was Copyright © 1998 Richard Braakman (also licensed under the
 # GPL 2 or higher)
 #
 # This program is free software; you can redistribute it and/or modify
@@ -24,35 +24,40 @@
 
 package Lintian::fields::architecture;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
-use Moo;
+use List::Compare;
+use List::MoreUtils qw(any);
 
 use Lintian::Architecture qw(:all);
 
 use constant EMPTY => q{};
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
 
-has architecture => (is => 'rwp', default => EMPTY);
-has have_r_package_not_arch_all => (is => 'rwp', default => 0);
+with 'Lintian::Check';
+
+has architecture => (is => 'rw', default => EMPTY);
+has have_r_package_not_arch_all => (is => 'rw', default => 0);
 
 sub setup {
     my ($self) = @_;
 
-    my $unsplit = $self->info->unfolded_field('architecture');
+    my $unsplit = $self->processable->unfolded_field('architecture');
 
     return
       unless defined $unsplit;
 
-    my @list = split(m/ /o, $unsplit);
+    my @architectures = split(m/ /o, $unsplit);
 
     return
-      unless @list;
+      unless @architectures;
 
-    $self->_set_architecture($list[0]);
+    $self->architecture($architectures[0]);
 
     return;
 }
@@ -60,11 +65,11 @@ sub setup {
 sub files {
     my ($self, $file) = @_;
 
-    $self->_set_have_r_package_not_arch_all(1)
+    $self->have_r_package_not_arch_all(1)
       if $file->name =~ m,^usr/lib/R/.*/DESCRIPTION,
       && !$file->is_dir
       && $self->package =~ /^r-(?:cran|bioc|other)-/
-      && $file->file_contents =~ m/NeedsCompilation: no/m
+      && $file->bytes =~ m/NeedsCompilation: no/m
       && $self->architecture ne 'all';
 
     return;
@@ -79,30 +84,30 @@ sub breakdown {
     return;
 }
 
-sub binary {
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
-    my $info = $self->info;
+    my $processable = $self->processable;
 
-    my $unsplit = $info->unfolded_field('architecture');
+    my $unsplit = $processable->unfolded_field('architecture');
 
     return
       unless defined $unsplit;
 
-    my @list = split(m/ /o, $unsplit);
+    my @architectures = split(m/ /o, $unsplit);
 
     return
-      unless @list;
+      unless @architectures;
 
-    for my $architecture (@list) {
+    for my $architecture (@architectures) {
         $self->tag('arch-wildcard-in-binary-package', $architecture)
           if is_arch_wildcard($architecture);
     }
 
-    $self->tag('too-many-architectures') if @list > 1;
+    $self->tag('too-many-architectures') if @architectures > 1;
 
-    my $architecture = $list[0];
+    my $architecture = $architectures[0];
 
     return
       if $architecture eq 'all';
@@ -120,53 +125,56 @@ sub always {
     my ($self) = @_;
 
     my $type = $self->type;
-    my $info = $self->info;
+    my $processable = $self->processable;
 
-    my $architecture = $info->unfolded_field('architecture');
+    my $value = $processable->unfolded_field('architecture');
 
-    unless (defined $architecture) {
+    unless (defined $value) {
         $self->tag('no-architecture-field');
         return;
     }
 
-    my @list = split(m/ /o, $architecture);
+    my @architectures = split(/ /, $value);
 
-    for my $arch (@list) {
+    for my $architecture (@architectures) {
 
-        $self->tag('unknown-architecture', $arch)
-          unless is_arch_or_wildcard($arch);
+        $self->tag('unknown-architecture', $architecture)
+          unless is_arch_or_wildcard($architecture)
+          || ($architecture eq 'source'
+            && ($type eq 'changes' || $type eq 'buildinfo'));
     }
 
-    if (@list > 1) {    # Check for magic architecture combinations.
+    # check for magic architecture combinations
+    if (@architectures > 1) {
 
-        my %archmap;
-        my $magic = 0;
+        my $magic_error = 0;
 
-        $archmap{$_}++ for (@list);
+        if (any { $_ eq 'all' } @architectures) {
+            $magic_error++
+              unless any { $type eq $_ } ('source', 'changes', 'buildinfo');
+        }
 
-        $magic++
-          if $type ne 'source' && $archmap{'all'};
+        my $anylc = List::Compare->new(\@architectures, ['any']);
+        if ($anylc->get_intersection) {
 
-        if ($archmap{'any'}) {
-
-            delete $archmap{'any'};
+            my @errorset = $anylc->get_Lonly;
 
             # Allow 'all' to be present in source packages as well
             # (#626775)
-            delete $archmap{'all'}
-              if $type eq 'source';
+            @errorset = grep { $_ ne 'all' } @errorset
+              if any { $type eq $_ } ('source', 'changes', 'buildinfo');
 
-            $magic++
-              if %archmap;
+            $magic_error++
+              if @errorset;
         }
 
-        $self->tag('magic-arch-in-arch-list') if $magic;
+        $self->tag('magic-arch-in-arch-list') if $magic_error;
     }
 
     # Used for later tests.
     my $arch_indep = 0;
     $arch_indep = 1
-      if @list == 1 && $list[0] eq 'all';
+      if @architectures == 1 && $architectures[0] eq 'all';
 
     return;
 }
