@@ -1,9 +1,9 @@
 # fields/built-using -- lintian check script (rewrite) -*- perl -*-
 #
-# Copyright (C) 2004 Marc Brockschmidt
+# Copyright © 2004 Marc Brockschmidt
 #
 # Parts of the code were taken from the old check script, which
-# was Copyright (C) 1998 Richard Braakman (also licensed under the
+# was Copyright © 1998 Richard Braakman (also licensed under the
 # GPL 2 or higher)
 #
 # This program is free software; you can redistribute it and/or modify
@@ -24,27 +24,29 @@
 
 package Lintian::fields::built_using;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
-use Moo;
-
 use Lintian::Relation qw(:constants);
 use Lintian::Util qw($PKGNAME_REGEX $PKGVERSION_REGEX);
 
-with('Lintian::Check');
-
 use constant {
     BUILT_USING_REGEX => qr/^$PKGNAME_REGEX \(= $PKGVERSION_REGEX\)$/o,
 };
 
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
+
 sub always {
     my ($self) = @_;
 
-    my $info = $self->info;
+    my $processable = $self->processable;
 
-    my $built_using = $info->field('built-using');
+    my $built_using = $processable->field('built-using');
 
     return
       unless defined $built_using;
