@@ -1,7 +1,8 @@
 # fields/standards-version -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
-# Copyright (C) 2008-2009 Russ Allbery
+# Copyright © 1998 Christian Schwarz and Richard Braakman
+# Copyright © 2008-2009 Russ Allbery
+# Copyright © 2020 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,8 +22,9 @@
 
 package Lintian::fields::standards_version;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Date::Parse qw(str2time);
@@ -30,7 +32,6 @@ use List::Util qw(first);
 use POSIX qw(strftime);
 
 use Lintian::Data;
-use Lintian::Util qw(internal_error);
 
 use Moo;
 use namespace::clean;
@@ -43,12 +44,12 @@ my $ANCIENT_DATE_DATA = Lintian::Data->new(
     qr{\s*<\s*},
     sub {
         my $date = str2time($_[1])
-          or internal_error("Cannot parse ANCIENT_DATE: $!");
+          or die "Cannot parse ANCIENT_DATE: $!";
         return $date;
     });
 
 my $ANCIENT_DATE = $ANCIENT_DATE_DATA->value('ANCIENT')
-  or internal_error('Cannot get ANCIENT_DATE');
+  or die 'Cannot get ANCIENT_DATE';
 
 my $STANDARDS= Lintian::Data->new('standards-version/release-dates', qr/\s+/o);
 
