@@ -1,6 +1,6 @@
 # files/privacy-breach -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
+# Copyright © 1998 Christian Schwarz and Richard Braakman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,15 +20,17 @@
 
 package Lintian::files::privacy_breach;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Lintian::SlidingWindow;
-use Lintian::Util qw(strip);
 
 use constant BLOCKSIZE => 16_384;
 
+use constant EMPTY => q{};
+
 use Moo;
 use namespace::clean;
 
@@ -39,7 +41,12 @@ my $PRIVACY_BREAKER_WEBSITES= Lintian::Data->new(
     qr/\s*\~\~/o,
     sub {
         my ($regex, $tag, $suggest) = split(/\s*\~\~\s*/, $_[1], 3);
-        $tag = defined($tag) ? strip($tag) : '';
+
+        $tag //= EMPTY;
+
+        # trim both ends
+        $tag =~ s/^\s+|\s+$//g;
+
         if (length($tag) == 0) {
             $tag = $_[0];
         }
@@ -271,7 +278,7 @@ sub check_tag_url_privacy_breach {
     # followed automatically.
     if(    $file->basename =~ '.xml$'
         && $tagattr eq 'link'
-        && $file->slurp=~ qr{ xmlns="http://projectmallard\.org/1\.0/"}) {
+        && $file->bytes=~ qr{ xmlns="http://projectmallard\.org/1\.0/"}) {
         return;
     }
 
