@@ -1,6 +1,6 @@
 # languages/java -- lintian check script -*- perl -*-
 
-# Copyright (C) 2011 Vincent Fourmond
+# Copyright © 2011 Vincent Fourmond
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,9 @@
 
 package Lintian::languages::java;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Basename;
@@ -38,7 +39,24 @@ with 'Lintian::Check';
 our $CLASS_REGEX = qr/\.(?:class|cljc?)/o;
 our $MAX_BYTECODE = Lintian::Data->new('java/constants', qr/\s*=\s*/o);
 
-sub always {
+sub source {
+    my ($self) = @_;
+
+    for my $file ($self->processable->patched->sorted_list) {
+        my $java_info = $file->java_info;
+        next
+          unless scalar keys %{$java_info};
+
+        my $files = $java_info->{files};
+        $self->tag('source-contains-prebuilt-java-object', $file)
+          if any { m/$CLASS_REGEX$/i } keys %{$files}
+          and $self->processable->name ne 'lintian';
+    }
+
+    return;
+}
+
+sub installable {
     my ($self) = @_;
 
     my $pkg = $self->package;
@@ -50,20 +68,6 @@ sub always {
     my $has_jars = 0;
     my $jmajlow = '-';
 
-    if ($type eq 'source') {
-        for my $file ($processable->patched->sorted_list) {
-            my $java_info = $file->java_info;
-            next
-              unless scalar keys %{$java_info};
-
-            my $files = $java_info->{files};
-            $self->tag('source-contains-prebuilt-java-object', $file)
-              if any { m/$CLASS_REGEX$/i } keys %{$files}
-              and $processable->name ne 'lintian';
-        }
-        return;
-    }
-
     my $depends = $processable->relation('strong')->unparse;
     # Remove all libX-java-doc packages to avoid thinking they are java libs
     #  - note the result may not be a valid dependency listing
