@@ -1,6 +1,6 @@
 # languages/javascript/nodejs -- lintian check script -*- perl -*-
 
-# Copyright (C) 2019-2020, Xavier Guimard <yadd@debian.org>
+# Copyright © 2019-2020, Xavier Guimard <yadd@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,9 @@
 
 package Lintian::languages::javascript::nodejs;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use JSON::MaybeXS;
@@ -52,7 +53,7 @@ sub source {
         # Ensure test file contains something
         if ($path and $path->is_open_ok) {
             $self->tag('pkg-js-autopkgtest-test-is-empty', $filename)
-              unless any { /^[^#]*\w/m } $path->slurp;
+              unless any { /^[^#]*\w/m } $path->bytes;
         } else {
             $self->tag('pkg-js-autopkgtest-test-is-missing', $filename);
         }
@@ -105,7 +106,7 @@ sub source {
         # Ensure test file contains something
         if ($path) {
             $self->tag('pkg-js-tools-test-is-empty', $filename)
-              unless any { /^[^#]*\w/m } $path->slurp;
+              unless any { /^[^#]*\w/m } $path->bytes;
         } else {
             $self->tag('pkg-js-tools-test-is-missing', $filename);
         }
@@ -157,7 +158,7 @@ sub files {
     my $provides
       = Lintian::Relation->and($processable->relation('provides'), $declared);
 
-    my $content = $file->slurp;
+    my $content = $file->bytes;
 
     # Look only valid package.json files
     my $pac;
