@@ -2,6 +2,7 @@
 
 # Copyright © 1998 Christian Schwarz and Richard Braakman
 # Copyright © 2020 Felix Lechner
+# Copyright © 2018, 2020 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,8 +22,9 @@
 
 package Lintian::md5sums;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use List::Compare;
@@ -56,7 +58,7 @@ sub files {
     return;
 }
 
-sub breakdown {
+sub binary {
     my ($self) = @_;
 
     my $control = $self->processable->control->lookup('md5sums');
@@ -71,20 +73,10 @@ sub breakdown {
 
         $self->tag('no-md5sums-control-file')
           unless $self->only_conffiles;
+
+        return;
     }
 
-    return;
-}
-
-sub binary {
-    my ($self) = @_;
-
-    my $control = $self->processable->control->lookup('md5sums');
-
-    # Is there an md5sums control file?
-    return
-      unless $control;
-
     # The md5sums file should not be a symlink.  If it is, the best
     # we can do is to leave it alone.
     return
@@ -97,7 +89,8 @@ sub binary {
     return
       if $control->size == 0;
 
-    my $text = $control->slurp;
+    my $text = $control->bytes;
+
     my ($md5sums, $errors) = read_md5sums($text);
 
     $self->tag('malformed-md5sums-control-file', $_)for @{$errors};
