@@ -1,6 +1,7 @@
 # shared-libs -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz
+# Copyright © 1998 Christian Schwarz
+# Copyright © 2018-2019 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +21,9 @@
 
 package Lintian::shared_libs;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use File::Basename;
@@ -29,7 +31,6 @@ use List::MoreUtils qw(any none uniq);
 
 use Lintian::Data;
 use Lintian::Relation;
-use Lintian::Util qw(strip);
 
 use Moo;
 use namespace::clean;
@@ -671,7 +672,7 @@ sub installable {
     # 6th step: check pre- and post- control files
     if (my $preinst = $processable->control->resolve_path('preinst')) {
         if ($preinst->is_open_ok) {
-            if ($preinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($preinst->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 $self->tag(
                     'maintscript-calls-ldconfig', 'preinst'
                       # Assume it is needed if glibc does it
@@ -684,7 +685,7 @@ sub installable {
     if (my $postinst = $processable->control->resolve_path('postinst')) {
         if ($postinst->is_open_ok) {
             # Decide if we call ldconfig
-            if ($postinst->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($postinst->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 if ($type eq 'udeb') {
                     $self->tag('udeb-postinst-must-not-call-ldconfig');
                 } else {
@@ -702,7 +703,10 @@ sub installable {
             # Determine if the package had an ldconfig trigger
             open(my $fd, '<', $triggers->unpacked_path);
             while (my $line = <$fd>) {
-                strip($line);
+
+                # trim both ends
+                $line =~ s/^\s+|\s+$//g;
+
                 $line =~ tr/ \t/ /s;
                 if ($line eq 'activate-noawait ldconfig') {
                     $we_trigger_ldconfig=1;
@@ -731,7 +735,7 @@ sub installable {
 
     if (my $prerm = $processable->control->resolve_path('prerm')) {
         if ($prerm->is_open_ok) {
-            if ($prerm->slurp =~ m/^[^\#]*\bldconfig\b/m) {
+            if ($prerm->bytes =~ m/^[^\#]*\bldconfig\b/m) {
                 $self->tag(
                     'maintscript-calls-ldconfig', 'prerm'
                       # Assume it is needed if glibc does it
@@ -742,7 +746,7 @@ sub installable {
 
     if (my $postrm = $processable->control->resolve_path('postrm')) {
         if ($postrm->is_open_ok) {
-            my $contents = $postrm->slurp;
+            my $contents = $postrm->bytes;
 
             # Decide if we call ldconfig
             if ($contents =~ m/^[^\#]*\bldconfig\b/m) {
