@@ -1,6 +1,7 @@
 # testsuite -- lintian check script -*- perl -*-
 
-# Copyright (C) 2013 Nicolas Boulenguez <nicolas@debian.org>
+# Copyright © 2013 Nicolas Boulenguez <nicolas@debian.org>
+# Copyright © 2017-2020 Chris Lamb <lamby@debian.org>
 
 # This file is part of lintian.
 
@@ -19,20 +20,19 @@
 
 package Lintian::testsuite;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
+use Path::Tiny;
+
 use Lintian::Data;
 use Lintian::Deb822Parser qw(
   DCTRL_COMMENTS_AT_EOL
-  read_dpkg_control
+  parse_dpkg_control_string
 );
 use Lintian::Relation;
-use Lintian::Util qw(
-  file_is_encoded_in_non_utf8
-  strip
-);
 
 use Moo;
 use namespace::clean;
@@ -49,6 +49,9 @@ my $KNOWN_OBSOLETE_RESTRICTIONS
   = Lintian::Data->new('testsuite/known-obsolete-restrictions');
 my $KNOWN_TESTSUITES = Lintian::Data->new('testsuite/known-testsuites');
 
+our $PYTHON3_ALL_DEPEND
+  = 'python3-all:any | python3-all-dev:any | python3-all-dbg:any';
+
 my %KNOWN_SPECIAL_DEPENDS = map { $_ => 1 } qw(
   @
   @builddeps@
@@ -83,15 +86,13 @@ sub source {
     if (defined($control)) {
         if (not $control->is_regular_file) {
             die 'debian tests control is not a regular file';
-        } elsif ($control->is_open_ok) {
-            my $path = $control->unpacked_path;
-            my $not_utf8_line = file_is_encoded_in_non_utf8($path);
 
-            if ($not_utf8_line) {
-                $self->tag('debian-tests-control-uses-national-encoding',
-                    "at line $not_utf8_line");
-            }
-            $self->check_control_contents($path);
+        } elsif ($control->is_valid_utf8) {
+            my $contents = $control->decoded_utf8;
+            $self->check_control_contents($contents);
+
+        } else {
+            $self->tag('debian-tests-control-uses-national-encoding');
         }
 
         $self->tag('unnecessary-testsuite-autopkgtest-field')
@@ -109,15 +110,16 @@ sub source {
 }
 
 sub check_control_contents {
-    my ($self, $path) = @_;
+    my ($self, $contents) = @_;
 
     my $processable = $self->processable;
 
     my (@paragraphs, @lines);
-    if (
-        not eval {
+    unless (
+        eval {
             @paragraphs
-              = read_dpkg_control($path, DCTRL_COMMENTS_AT_EOL, \@lines);
+              = parse_dpkg_control_string($contents, DCTRL_COMMENTS_AT_EOL,
+                \@lines);
         }
     ) {
         chomp $@;
@@ -172,8 +174,12 @@ sub check_control_paragraph {
     }
 
     if (exists $paragraph->{'features'}) {
-        my $features = strip($paragraph->{'features'});
+        my $features = $paragraph->{'features'};
         for my $feature (split(/\s*,\s*|\s+/ms, $features)) {
+
+            next
+              unless length $feature;
+
             if (not exists $KNOWN_FEATURES{$feature}
                 and $feature !~ m/^test-name=\S+/) {
                 $self->tag(
@@ -185,8 +191,10 @@ sub check_control_paragraph {
     }
 
     if (exists $paragraph->{'restrictions'}) {
-        my $restrictions = strip($paragraph->{'restrictions'});
+        my $restrictions = $paragraph->{'restrictions'};
         for my $restriction (split(/\s*,\s*|\s+/ms, $restrictions)) {
+            next
+              unless length $restriction;
             $self->tag(
                 'unknown-runtime-tests-restriction', $restriction,
                 'paragraph starting at line', $line
@@ -198,17 +206,23 @@ sub check_control_paragraph {
     }
 
     if (exists $paragraph->{'tests'}) {
-        my $tests = strip($paragraph->{'tests'});
+        my $tests = $paragraph->{'tests'};
         my $directory = 'debian/tests';
         if (exists $paragraph->{'tests-directory'}) {
             $directory = $paragraph->{'tests-directory'};
         }
         for my $testname (split(/\s*,\s*|\s+/ms, $tests)) {
+
+            next
+              unless length $testname;
+
             $self->check_test_file($directory, $testname, $line);
         }
     }
     if (exists($paragraph->{'depends'})) {
-        my $dep = Lintian::Relation->new(strip($paragraph->{'depends'}));
+        my $depends = $paragraph->{'depends'};
+        $depends =~ s/^\s+|\s+$//g;
+        my $dep = Lintian::Relation->new($depends);
         for my $unparsable ($dep->unparsable_predicates) {
             # @ is not a valid predicate in general, but autopkgtests
             # allows it.
@@ -251,8 +265,15 @@ sub check_test_file {
             $self->tag('uses-deprecated-adttmp', $path, "(line $.)")
               if $x =~ m/ADTTMP/;
             $self->tag('runtime-test-file-uses-installed-python-versions',
-                $path, "(line $.)")
-              if $x =~ m/py3versions\s+(?:--installed|-i)/;
+                $path, "$1", "(line $.)")
+              if $x =~ m/(py3versions\s+([\w\-\s]*--installed|-\w*i\w*))/;
+            #<<< no Perl tidy
+            $self->tag(
+                'runtime-test-file-uses-supported-python-versions-without-python-all-build-depends',
+                $path, "$1", "(line $.)"
+            ) if $x =~ m/(py3versions\s+([\w\-\s]*--supported|-\w*s\w*))/
+              and not $processable->relation_noarch('build-depends-all')->implies($PYTHON3_ALL_DEPEND);
+            #>>>
         }
         close($fd);
     }
