@@ -2,6 +2,8 @@
 #
 # reporting-sync-state
 #
+# Copyright © 2018 Chris Lamb <lamby@debian.org>
+#
 # This program is free software.  It is distributed under the terms of
 # the GNU General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any
@@ -18,8 +20,9 @@
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Getopt::Long();
@@ -36,7 +39,6 @@ use Lintian::Reporting::Util qw(
 );
 use Lintian::Util qw(
   open_gz
-  strip
 );
 
 my $DEFAULT_CHECKSUM = 'sha256';
@@ -366,7 +368,10 @@ sub _parse_srcs_pg {
     return unless $ACTIVE_GROUPS{$group_id};
     $dir .= '/' if $dir;
     foreach my $f (split m/\n/, $paragraph->{"checksums-${DEFAULT_CHECKSUM}"}){
-        strip($f);
+
+        # trim both ends
+        $f =~ s/^\s+|\s+$//g;
+
         next unless $f && $f =~ m/\.dsc$/;
         my ($checksum, undef, $basename) = split(m/\s++/, $f);
         my $b64_checksum = encode_base64(pack('H*', $checksum));
