@@ -41,7 +41,7 @@ use Lintian::Processable::Buildinfo;
 use Lintian::Processable::Changes;
 use Lintian::Processable::Source;
 use Lintian::Processable::Udeb;
-use Lintian::Util qw(get_dsc_info_from_string strip human_bytes);
+use Lintian::Util qw(get_dsc_info_from_string human_bytes);
 
 use constant EMPTY => q{};
 use constant SPACE => q{ };
@@ -135,8 +135,6 @@ Hash with active jobs.
 
 =item C<saved_spelling_exceptions>
 
-=item C<shared_storage>
-
 =cut
 
 has pooldir => (is => 'rw', default => EMPTY);
@@ -159,8 +157,6 @@ has saved_direct_dependencies => (is => 'rw', default => sub { {} });
 has saved_direct_reliants => (is => 'rw', default => sub { {} });
 has saved_spelling_exceptions => (is => 'rw', default => sub { {} });
 
-has shared_storage => (is => 'rw', default => sub { {} });
-
 =item Lintian::Group->init_from_file (FILE)
 
 Add all processables from .changes or .buildinfo file FILE.
@@ -243,12 +239,13 @@ sub init_from_file {
         $dir =~ s,(.+)/[^/]+$,$1,;
     }
     my $key = $type eq 'buildinfo' ? 'checksums-sha256' : 'files';
-    for my $line (split(/\n/o, $info->{$key}//'')) {
+    for my $line (split(/\n/, $info->{$key}//'')) {
 
         next
           unless defined $line;
 
-        strip($line);
+        # trim both ends
+        $line =~ s/^\s+|\s+$//g;
 
         next
           if $line eq EMPTY;
@@ -265,10 +262,8 @@ sub init_from_file {
         next
           if !$file || $file =~ m,/,;
 
-        unless (-f "$dir/$file") {
-            print STDERR "$dir/$file does not exist, exiting\n";
-            exit 2;
-        }
+        die "$dir/$file does not exist, exiting\n"
+          unless -f "$dir/$file";
 
         # only care about some files; ddeb is ubuntu dbgsym
         next
@@ -301,7 +296,10 @@ sub unpack {
         if ($processable->type eq 'source') {
             my (undef, $dir, undef)= File::Spec->splitpath($processable->path);
             for my $fs (split(m/\n/o, $processable->field('files'))) {
-                strip($fs);
+
+                # trim both ends
+                $fs =~ s/^\s+|\s+$//g;
+
                 next if $fs eq '';
                 my @t = split(/\s+/o,$fs);
                 next if ($t[2] =~ m,/,o);
@@ -329,35 +327,23 @@ Process group.
 =cut
 
 sub process {
-    my ($self, $exit_code_ref, $override_count, $opt, $OUTPUT)= @_;
+    my ($self, $ignored_overrides, $option, $OUTPUT)= @_;
 
     $self->processing_start(gmtime->datetime);
 
-    my $all_ok = 1;
+    my $success = 1;
 
     my $timer = [gettimeofday];
 
-    my %statistics = (
-        types     => {},
-        severity  => {},
-        tags      => {},
-        overrides => {},
-    );
-
-    my @reported_tags;
-
     for my $processable ($self->get_processables){
 
         my $declared_overrides;
         my %used_overrides;
 
-        # to store tag names as well
-        my $override_count->{ignored} = {};
-
         $OUTPUT->debug_msg(1,
             'Base directory for group: ' . $processable->groupdir);
 
-        unless ($opt->{'no-override'}) {
+        unless ($option->{'no-override'}) {
 
             $OUTPUT->debug_msg(1, 'Loading overrides file (if any) ...');
 
@@ -398,7 +384,7 @@ sub process {
 
                 unless ($self->profile->is_overridable($tagname)) {
                     delete $declared_overrides->{$tagname};
-                    $override_count->{ignored}{$tagname}++;
+                    $ignored_overrides->{$tagname}++;
                 }
             }
 
@@ -439,8 +425,7 @@ sub process {
                 print STDERR "internal error: cannot run $checkname check",
                   " on package $procid\n";
                 $OUTPUT->warning("skipping check of $procid");
-                $$exit_code_ref = 2;
-                $all_ok = 0;
+                $success = 0;
 
                 next;
             }
@@ -521,99 +506,14 @@ sub process {
 
                 # cannot be overridden or suppressed
                 $processable->tag('unused-override', $tagname, $hint);
-
-                $override_count->{unused}++;
             }
         }
 
-        # associate all tags with processable
-        $_->processable($processable) for @{$processable->tags};
-
-        push(@reported_tags, @{$processable->tags});
-
-        # remove circular references
-        $processable->tags([]);
+        # copy tag specifications into tags
+        $_->info($self->profile->get_taginfo($_->name))
+          for @{$processable->tags};
     }
 
-    # copy tag specifications into tags
-    $_->info($self->profile->get_taginfo($_->name)) for @reported_tags;
-
-    for my $tag (@reported_tags) {
-
-        my $record = \%statistics;
-        $record = $statistics{overrides}
-          if $tag->override;
-
-        $record->{tags}{$tag->name}++;
-        $record->{severity}{$tag->info->effective_severity}++;
-
-        my $code = $tag->info->code;
-        $code = 'X' if $tag->info->experimental;
-        $record->{types}{$code}++;
-    }
-
-    unless ($$exit_code_ref) {
-        if ($statistics{types}{E}) {
-            $$exit_code_ref = 1;
-        }
-    }
-
-    # Report override statistics.
-    unless ($opt->{'no-override'} || $opt->{'show-overrides'}) {
-
-        my $errors = $statistics{overrides}{types}{E} // 0;
-        my $warnings = $statistics{overrides}{types}{W} // 0;
-        my $info = $statistics{overrides}{types}{I} // 0;
-
-        $override_count->{errors} += $errors;
-        $override_count->{warnings} += $warnings;
-        $override_count->{info} += $info;
-    }
-
-    # discard disabled tags
-    @reported_tags
-      = grep { $self->profile->tag_is_enabled($_->name) } @reported_tags;
-
-    # discard experimental tags
-    @reported_tags = grep { !$_->info->experimental } @reported_tags
-      unless $opt->{'display-experimental'};
-
-    # discard overridden tags
-    @reported_tags = grep { !defined $_->override } @reported_tags
-      unless $opt->{'show-overrides'};
-
-    # discard outside the selected display level
-    @reported_tags
-      = grep { $self->profile->display_level_for_tag($_->name) }@reported_tags;
-
-    my $reference_limit = $opt->{'display-source'} // [];
-    if (@{$reference_limit}) {
-
-        my @topic_tags;
-        for my $tag (@reported_tags) {
-            my @references = split(/,/, $tag->info->references);
-
-            # retain the first word
-            s/^([\w-]+)\s.*/$1/ for @references;
-
-            # remove anything in parentheses at the end
-            s/\(\S+\)$// for @references;
-
-            # check if tag refers to the selected references
-            my $referencelc
-              = List::Compare->new(\@references, $reference_limit);
-            next
-              unless $referencelc->get_intersection;
-
-            push(@topic_tags, $tag);
-        }
-
-        @reported_tags = @topic_tags;
-    }
-
-    # put tags back into their respective processables
-    push(@{$_->processable->tags}, $_) for @reported_tags;
-
     $self->processing_end(gmtime->datetime);
 
     my $raw_res = tv_interval($timer);
@@ -622,7 +522,7 @@ sub process {
         'Checking all of group ' . $self->name . " done ($tres)");
     $OUTPUT->perf_log($self->name . ",total-group-check,${raw_res}");
 
-    if ($opt->{'debug'} > 2) {
+    if ($option->{'debug'} > 2) {
 
         # suppress warnings without reliable sizes
         $Devel::Size::warn = 0;
@@ -642,7 +542,7 @@ sub process {
 
     $self->clean_lab($OUTPUT);
 
-    return $all_ok;
+    return $success;
 }
 
 =item clean_lab
@@ -748,8 +648,6 @@ sub add_processable{
         $self->$type->{$id} = $processable;
     }
 
-    $processable->shared_storage($self->shared_storage);
-
     return 1;
 }
 
