@@ -1,7 +1,7 @@
 # -*- perl -*-
 # Lintian::Internal::FrontendUtil -- internal helpers for lintian frontends
 
-# Copyright (C) 2011 Niels Thykier
+# Copyright © 2011 Niels Thykier
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -18,8 +18,9 @@
 
 package Lintian::Internal::FrontendUtil;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Exporter qw(import);
@@ -27,7 +28,6 @@ use Exporter qw(import);
 use Carp qw(croak);
 use Dpkg::Vendor;
 
-use Lintian::CollScript;
 use Lintian::Util qw(check_path safe_qx);
 
 our @EXPORT_OK= qw(check_test_feature default_parallel split_tag
